/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 19/09/2001
**/
#include <qpainter.h>
#include <ctype.h>
#include "quad.h"

Quad::Quad( QWidget *parent, const char *name )
        : QGLWidget( QGLFormat( SingleBuffer|NoDepthBuffer ), parent, name )
{
	signo = 1;	

	// setMouseTracking( TRUE );

	setFocusPolicy(QWidget::StrongFocus);

	timer = new QTimer( this, "mytimer" );
	connect ( timer, SIGNAL(timeout()), this, SLOT(timerDone()) );

	xrot = yrot = 0;
	angle = 0;
}


void Quad::timerDone( )
{
	angle += 10.0*signo;

	if ( angle >= 360.0 ) angle = 0.0;
	if ( angle <= -360.0 ) angle = 0.0;
	
	updateGL( );
}

void Quad::mouseMoveEvent( QMouseEvent * e )
{
	QPoint c = e->pos( );

	xmouse = c.x( );
	ymouse = c.y( );
	if ( xmouse>=0 && ymouse>=0 )  emit changePos ( );

	yrot = (float)xmouse*360.f/(float)windowWidth - 180.f;
    xrot = (float)ymouse*360.f/(float)windowHeight - 180.f;

	updateGL( );
}

void Quad::keyPressEvent ( QKeyEvent * e )
{
	switch ( tolower(e->ascii()) ) {
		case '+':   
			signo = 1;
			break;
		case '-':  
			signo = -1;
			break;
	}
}

//
// This method puts the initial coordinates each time window is resized
//
void Quad::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	windowWidth = w;
	windowHeight = h;
	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	if( w <= h )
		glOrtho ( -1.0, 1.0, -1.0/aspect, 1.0/aspect, -1.0, 1.0);
	else
		glOrtho (-aspect, aspect, -1.0, 1.0, -1.0, 1.0);
	
	emit changeSize ( );
}

QSizePolicy Quad::sizePolicy() const
{
	return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

void Quad::paintGL(void)
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();             /* clear the matrix */
	glRotatef (xrot, 1.0, 0.0, 0.0);
	glRotatef (yrot, 0.0, 1.0, 0.0);

	/* viewing transformation  */
	glPushMatrix( );
	glScalef ( 0.7, 0.7, 0.7);      /* modeling transformation */ 

	glRotatef ( angle , 0.0, 0.0, 1.0);

	/** Draw simply four lines. These form a quadrilater **/
	glBegin(GL_LINE_LOOP);
		glColor3f( 0.0, 0.0, 1.0 );
		glVertex2i( 1, 1 );
		glVertex2i( 1,-1 );
		glVertex2i(-1,-1 );
		glVertex2i(-1, 1 );
	glEnd();
	
	glPopMatrix( );

	glFlush( );
}

void Quad::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1.0, 1.0,-1.0, 1.0,-1.0, 1.0);
}

